@@ -1,5 +1,5 @@
-/* $Id: error.c,v 1.23 2004/10/20 21:53:08 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2004 */
+/* $Id: error.c,v 1.26 2005/03/18 20:21:55 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* the logging subsystem */
@@ -43,11 +43,12 @@ struct diag_opts {
    FILE *logfile;
    int logfacility;
    bool micros;
+   int exitstatus;	/* pass signal number to error exit */
 } ;
 
 
 struct diag_opts diagopts =
-  { NULL, E_ERROR, E_ERROR, 1, 0, NULL, LOG_DAEMON, false } ;
+  { NULL, E_ERROR, E_ERROR, 1, 0, NULL, LOG_DAEMON, false, 0 } ;
 
 static void _msg(int level, const char *buff, const char *syslp);
 
@@ -112,10 +113,17 @@ void diag_set(char what, const char *arg) {
       openlog(diagopts.progname, LOG_PID, diagopts.logfacility);
       break;
    case 'd': --diagopts.msglevel; break;
-   case 'D': diagopts.msglevel = strtoul(arg, NULL, 10); break;
-   case 'e': diagopts.exitlevel = strtoul(arg, NULL, 10); break;
    case 'u': diagopts.micros = true; break;
-   default: msg(E_ERROR, "unknown log device specification %c%s", what, arg);
+   default: msg(E_ERROR, "unknown diagnostic option %c", what);
+   }
+}
+
+void diag_set_int(char what, int arg) {
+   switch (what) {
+   case 'D': diagopts.msglevel = arg; break;
+   case 'e': diagopts.exitlevel = arg; break;
+   case 'x': diagopts.exitstatus = arg; break;
+   default: msg(E_ERROR, "unknown diagnostic option %c", what);
    }
 }
 
@@ -193,11 +201,13 @@ void msg(int level, const char *format, ...) {
 #endif
    }
 #endif /* !HAVE_GETTIMEOFDAY */
-   sprintf(buff+bytes, "%s["F_pid"] ", diagopts.progname, getpid());
-   syslp = strchr(buff+23, '\0');
-   sprintf(syslp, "%c ", "DINWEF"[level]);
-   bufp = strchr(syslp, '\0');
-   vsnprintf(bufp, BUFLEN-35-1, format, ap);
+   bufp = buff + bytes;
+   bytes = sprintf(bufp, "%s["F_pid"] ", diagopts.progname, getpid());
+   bufp += bytes;
+   syslp = bufp;
+   *bufp++ = "DINWEF"[level];
+   *bufp++ = ' ';
+   vsnprintf(bufp, BUFLEN-(bufp-buff)-1, format, ap);
    strcat(bufp, "\n");
    _msg(level, buff, syslp);
    if (level >= diagopts.exitlevel) {
@@ -206,7 +216,7 @@ void msg(int level, const char *format, ...) {
 	 sprintf(syslp, "N exit(1)\n");
 	 _msg(E_NOTICE, buff, syslp);
       }
-      exit(1);
+      exit(diagopts.exitstatus ? diagopts.exitstatus : 1);
    }
    va_end(ap);
 }
