@@ -1,4 +1,4 @@
-/* $Id: filan.c,v 1.30 2004/09/21 19:11:00 gerhard Exp $ */
+/* $Id: filan.c,v 1.36 2004/12/03 22:40:07 gerhard Exp $ */
 /* Copyright Gerhard Rieger 2001-2004 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
@@ -30,27 +30,76 @@ struct sockopt {
    libwrap */
 int allow_severity, deny_severity;
 
+/* global variables for configuring filan */
+bool filan_followsymlinks;
+bool filan_rawoutput;
+
 
 int sockoptan(int fd, const struct sockopt *optname, int socklay, FILE *outfile);
 int tcpan(int fd, FILE *outfile);
 const char *getfiletypestring(int st_mode);
 
+static int printtime(FILE *outfile, time_t time);
 
-int filan(int fd, FILE *outfile) {
-   static int headprinted;
-   int result, cloexec, flags;
-#if defined(F_GETOWN)
-   int sigown;
-#endif
-#if defined(F_GETSIG)
-   int sigio;
-#endif /* defined(F_GETSIG) */
+static int headprinted;
+
+int filan_file(const char *filename, FILE *outfile) {
+   int fd = -1;
+   int result;
 #if HAVE_STAT64
    struct stat64 buf = {0};
 #else
    struct stat buf = {0};
 #endif /* !HAVE_STAT64 */
 
+   if (filan_followsymlinks) {
+#if HAVE_STAT64
+      result = Stat64(filename, &buf);
+#else
+      result = Stat(filename, &buf);
+#endif /* !HAVE_STAT64 */
+      if (result < 0) {
+	 Error3("stat(\"%s\", %p): %s", filename, &buf, strerror(errno));
+      }
+   } else {
+#if HAVE_STAT64
+      result = Lstat64(filename, &buf);
+#else
+      result = Lstat(filename, &buf);
+#endif /* !HAVE_STAT64 */
+      if (result < 0) {
+	 Error3("lstat(\"%s\", %p): %s", filename, &buf, strerror(errno));
+      }
+   }
+   switch (buf.st_mode&S_IFMT) {
+#ifdef S_IFSOCK
+   case S_IFSOCK: /* probably, it's useless to make a socket and describe it */
+      break;
+#endif /* S_IFSOCK */
+   default:
+      if ((fd =
+	   Open(filename,  O_RDONLY|O_NOCTTY|O_NONBLOCK
+#ifdef O_LARGEFILE
+		|O_LARGEFILE
+#endif
+		, 0700))
+	  < 0) {
+	 Warn2("open(\"%s\", O_RDONLY|O_NOCTTY|O_NONBLOCK|O_LARGEFILE, 0700): %s",
+	       filename, strerror(errno));
+      }
+   }
+     
+   return filan_stat(&buf, fd, -1, outfile);
+}
+
+int filan_fd(int fd, FILE *outfile) {
+#if HAVE_STAT64
+   struct stat64 buf = {0};
+#else
+   struct stat buf = {0};
+#endif /* !HAVE_STAT64 */
+   int result;
+
    Debug1("checking file descriptor %u", fd);
 #if HAVE_STAT64
    result = Fstat64(fd, &buf);
@@ -66,38 +115,72 @@ int filan(int fd, FILE *outfile) {
       return -1;
    }
    Debug2("fd %d is a %s", fd, getfiletypestring(buf.st_mode));
-   cloexec = Fcntl(fd, F_GETFD);
-   flags   = Fcntl(fd, F_GETFL);
-#if defined(F_GETOWN)
-   sigown  = Fcntl(fd, F_GETOWN);
-#endif
-#if defined(F_GETSIG)
-   sigio   = Fcntl(fd, F_GETSIG);
-#endif /* defined(F_GETSIG) */
+
+   return filan_stat(&buf, fd, fd, outfile);
+}
+
+
+int filan_stat(
+#if HAVE_STAT64
+	       struct stat64 *buf
+#else
+	       struct stat *buf
+#endif /* !HAVE_STAT64 */
+	       , int statfd, int dynfd, FILE *outfile) {
+   char stdevstr[8];
+   int result;
+
+   /* print header */
    if (!headprinted) {
-      fputs("  FD  type\tdevice\tinode\tmode\tlinks\tuid\tgid"
+      if (filan_rawoutput) {
+	 fputs("  FD  type\tdevice\tinode\tmode\tlinks\tuid\tgid"
 #if HAVE_ST_RDEV
-	    "\trdev"
+	       "\trdev"
 #endif
-	    "\tsize"
+	       "\tsize"
 #if HAVE_ST_BLKSIZE
-	    "\tblksize"
+	       "\tblksize"
 #endif
 #if HAVE_ST_BLOCKS
-	    "\tblocks"
+	       "\tblocks"
 #endif
-	    "\tatime\t\tmtime\t\tctime\t\tcloexec\tflags"
+	       "\tatime\t\tmtime\t\tctime\t\tcloexec\tflags"
 #if defined(F_GETOWN)
-	    "\tsigown"
+	       "\tsigown"
 #endif
-	    , outfile);
+	       , outfile);
+      } else /* !rawoutput */ {
+	 fputs("  FD  type\tdevice\tinode\tmode\tlinks\tuid\tgid"
+#if HAVE_ST_RDEV
+	       "\trdev"
+#endif
+	       "\tsize"
+#if HAVE_ST_BLKSIZE
+	       "\tblksize"
+#endif
+#if HAVE_ST_BLOCKS
+	       "\tblocks"
+#endif
+	       "\tatime\t\t\t\tmtime\t\t\t\tctime\t\t\t\tcloexec\tflags"
+#if defined(F_GETOWN)
+	       "\tsigown"
+#endif
+	       , outfile);
+
+      } /* endif !rawoutput */
+
 #if defined(F_GETSIG)
       fputs("\tsigio", outfile);
 #endif /* defined(F_GETSIG) */
       fputc('\n', outfile);
       headprinted = 1;
    }
-   fprintf(outfile, "%4d: %s\t"F_st_dev"\t"
+   if (filan_rawoutput) {
+      snprintf(stdevstr, 8, F_st_dev, buf->st_dev);
+   } else {
+      snprintf(stdevstr, 8, "%hu,%hu", (unsigned short)buf->st_dev>>8, (unsigned short)buf->st_dev&0xff);
+   }
+   fprintf(outfile, "%4d: %s\t%s\t"
 #if HAVE_STAT64
 	   F_st64_ino
 #else
@@ -123,23 +206,60 @@ int filan(int fd, FILE *outfile) {
 	   "\t"F_st_blocks
 #endif /* HAVE_STAT64 */
 #endif
-	   "\t"F_time"\t"F_time"\t"F_time,
-	   fd, getfiletypestring(buf.st_mode),
-	   buf.st_dev,
-	   buf.st_ino,
-	   buf.st_mode, buf.st_nlink, buf.st_uid,
-	   buf.st_gid,
+	   ,
+	   (dynfd>=0?dynfd:statfd), getfiletypestring(buf->st_mode),
+	   stdevstr,
+	   buf->st_ino,
+	   buf->st_mode, buf->st_nlink, buf->st_uid,
+	   buf->st_gid,
 #if HAVE_ST_RDEV
-	   (unsigned short)buf.st_rdev>>8, (unsigned short)buf.st_rdev&0xff,
+	   (unsigned short)buf->st_rdev>>8, (unsigned short)buf->st_rdev&0xff,
 #endif
-	   buf.st_size,
+	   buf->st_size
 #if HAVE_ST_BLKSIZE
-	   buf.st_blksize,
+	   , buf->st_blksize
 #endif
 #if HAVE_ST_BLOCKS
-	   buf.st_blocks,	/* on Linux, this applies to stat and stat64 */
+	   , buf->st_blocks	/* on Linux, this applies to stat and stat64 */
 #endif
-	   buf.st_atime, buf.st_mtime, buf.st_ctime);
+	   );
+
+      printtime(outfile, buf->st_atime);
+      printtime(outfile, buf->st_mtime);
+      printtime(outfile, buf->st_ctime);
+
+#if 0
+   {
+      fputc('\t', outfile);
+      time = asctime(localtime(&buf->st_mtime));
+      if (strchr(time, '\n'))  *strchr(time, '\n') = '\0';
+      fputs(time, outfile);
+
+      fputc('\t', outfile);
+      time = asctime(localtime(&buf->st_ctime));
+      if (strchr(time, '\n'))  *strchr(time, '\n') = '\0';
+      fputs(time, outfile);
+   }
+#endif
+
+   /* here comes dynamic info - it is only meaningful with preexisting FDs */
+  if (dynfd >= 0) {	/*!indent */
+   int cloexec, flags;
+#if defined(F_GETOWN)
+   int sigown;
+#endif
+#if defined(F_GETSIG)
+   int sigio;
+#endif /* defined(F_GETSIG) */
+
+   cloexec = Fcntl(dynfd, F_GETFD);
+   flags   = Fcntl(dynfd, F_GETFL);
+#if defined(F_GETOWN)
+   sigown  = Fcntl(dynfd, F_GETOWN);
+#endif
+#if defined(F_GETSIG)
+   sigio   = Fcntl(dynfd, F_GETSIG);
+#endif /* defined(F_GETSIG) */
    fprintf(outfile, "\t%d\tx%06x", cloexec, flags);
 #if defined(F_GETOWN)
    fprintf(outfile, "\t%d", sigown);
@@ -147,13 +267,24 @@ int filan(int fd, FILE *outfile) {
 #if defined(F_GETSIG)
    fprintf(outfile, "\t%d", sigio);
 #endif /* defined(F_GETSIG) */
+  } else {
+     fputs("\t\t"
+#if defined(F_GETOWN)
+	   "\t"
+#endif
+#if defined(F_GETSIG)
+	   "\t"
+#endif /* defined(F_GETSIG) */
+	   , outfile);
+  }
 
    /* now see for type specific infos */
-   switch (buf.st_mode&S_IFMT) {
+  if (statfd >= 0) { /*!indent */
+   switch (buf->st_mode&S_IFMT) {
    case (S_IFIFO):	/* 1, FIFO */
       break;
    case (S_IFCHR):	/* 2, character device */
-      result = cdevan(fd, outfile);
+      result = cdevan(statfd, outfile);
       break;
    case (S_IFDIR):	/* 4, directory */
       break;
@@ -166,7 +297,7 @@ int filan(int fd, FILE *outfile) {
 #ifdef S_IFSOCK
    case (S_IFSOCK): /* 12, socket */
 #if WITH_SOCKET
-      result = sockan(fd, outfile);
+      result = sockan(statfd, outfile);
 #else
       Error("SOCKET support not compiled in");
       return -1;
@@ -174,6 +305,7 @@ int filan(int fd, FILE *outfile) {
       break;
 #endif /* S_IFSOCK */
    }
+  }
    /* ioctl() */
    fputc('\n', outfile);
    return 0;
@@ -213,6 +345,7 @@ int cdevan(int fd, FILE *outfile) {
    if (ret > 0) {
       struct termios termarg;
       char *name;
+      int i;
 
       if ((name = Ttyname(fd)) == NULL) {
 	 /*Error2("ttyname(%d): %s", fd, strerror(errno));*/
@@ -226,6 +359,30 @@ int cdevan(int fd, FILE *outfile) {
       }
       fprintf(outfile, " \tIFLAGS=%08x OFLAGS=%08x CFLAGS=%08x LFLAGS=%08x",
 	      termarg.c_iflag, termarg.c_oflag, termarg.c_cflag, termarg.c_lflag);
+
+      /* and the control characters */
+      if (filan_rawoutput) {
+	 for (i=0; i<NCCS; ++i) {
+	    fprintf(outfile, " cc[%d]=%d", i, termarg.c_cc[i]);
+	 }
+      } else {
+	 for (i=0; i<NCCS; ++i) {
+	    int ch;
+	    unsigned char s[4];
+	    ch = termarg.c_cc[i];
+	    if (isprint(ch)) {
+	       s[0] = ch; s[1]= '\0';
+	    } else if (ch < ' ') {
+	       s[0] = '^'; s[1] = ch+'@'; s[2] = '\0';
+	    } else {
+	       s[0] = 'x';
+	       s[1] = (ch>>4)>=10?(ch>>4)-10+'A':(ch>>4)+'0';
+	       s[2] = (ch&0x0f)>=10?(ch&0x0f)-10+'A':(ch&0x0f)+'0';
+	       s[3] = '\0';
+	    }
+	    fprintf(outfile, " cc[%d]=%s", i, s);
+	 }
+      }
    }
    return 0;
 }
@@ -523,6 +680,39 @@ int tcpan(int fd, FILE *outfile) {
 #endif
 #ifdef TCP_QUICKACK
       { TCP_QUICKACK, "TCP_QUICKACK" },
+#endif
+#ifdef TCP_MD5SIG
+      { TCP_MD5SIG, "TCP_MD5SIG" },
+#endif
+#ifdef TCP_NOOPT
+      { TCP_NOOPT, "TCP_NOOPT" },
+#endif
+#ifdef TCP_NOPUSH
+      { TCP_NOPUSH, "TCP_NOPUSH" },
+#endif
+#ifdef TCP_SACK_DISABLE
+      { TCP_SACK_DISABLE, "TCP_SACK_DISABLE" },
+#endif
+#ifdef TCP_SIGNATURE_ENABLE
+      { TCP_SIGNATURE_ENABLE, "TCP_SIGNATURE_ENABLE" },
+#endif
+#ifdef TCP_ABORT_THRESHOLD
+      { TCP_ABORT_THRESHOLD, "TCP_ABORT_THRESHOLD" },
+#endif
+#ifdef TCP_CONN_ABORT_THRESHOLD
+      { TCP_CONN_ABORT_THRESHOLD, "TCP_CONN_ABORT_THRESHOLD" },
+#endif
+#ifdef TCP_KEEPINIT
+      { TCP_KEEPINIT, "TCP_KEEPINIT" },
+#endif
+#ifdef TCP_PAWS
+      { TCP_PAWS, "TCP_PAWS" },
+#endif
+#ifdef TCP_SACKENA
+      { TCP_SACKENA, "TCP_SACKENA" },
+#endif
+#ifdef TCP_TSOPTENA
+      { TCP_TSOPTENA, "TCP_TSOPTENA" },
 #endif
       {0, NULL}
    } ;
@@ -585,11 +775,10 @@ int isasocket(int fd) {
 #endif /* HAVE_STAT64 */
    retval =
 #if HAVE_STAT64
-      Fstat64
+      Fstat64(fd, &props);
 #else
-      Fstat
+      Fstat(fd, &props);
 #endif
-	  (fd, &props);
    if (retval < 0) {
       Info3("fstat(%d, %p): %s", fd, &props, strerror(errno));
       return 0;
@@ -616,3 +805,17 @@ const char *getfiletypestring(int st_mode) {
    }
    return s;
 }
+
+static int printtime(FILE *outfile, time_t time) {
+   const char *s;
+
+   if (filan_rawoutput) {
+      fprintf(outfile, "\t"F_time, time);
+   } else {
+      fputc('\t', outfile);
+      s = asctime(localtime(&time));
+      if (strchr(s, '\n'))  *strchr(s, '\n') = '\0';
+      fputs(s, outfile);
+   }
+   return 0;
+}   
