@@ -1,5 +1,5 @@
-/* $Id: procan_main.c,v 1.10 2004/12/11 13:28:05 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2004 */
+/* $Id: procan_main.c,v 1.11 2005/03/12 10:54:25 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 const char copyright[] = "procan by Gerhard Rieger - send bug reports to socat@dest-unreach.org";
@@ -41,15 +41,11 @@ int main(int argc, const char *argv[]) {
 #endif
       case '\0': break;
       default:
-	 {
-	    char l[2];
-	    l[0] = '0'+E_FATAL; l[1] = '\0';
-	    diag_set('e', l);
-	    Error1("unknown option \"%s\"", arg1[0]);
+	 diag_set_int('e', E_FATAL);
+	 Error1("unknown option \"%s\"", arg1[0]);
 #if WITH_HELP
-	    procan_usage(stderr);
+	 procan_usage(stderr);
 #endif
-	 }
 	 exit(1);
       }
       if (arg1[0][1] == '\0')
