@@ -1,5 +1,5 @@
-/* $Id: socat.c,v 1.90 2004/12/05 12:24:53 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2004 */
+/* $Id: socat.c,v 1.92 2005/03/13 12:12:26 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this is the main source, including command line option parsing, general
@@ -53,6 +53,7 @@ int socat(const char *address1, const char *address2);
 int _socat(void);
 int cv_newline(unsigned char **buff, ssize_t *bytes, int lineterm1, int lineterm2);
 void socat_signal(int sig);
+static int socat_sigchild(struct single *file);
 
 void lftocrlf(char **in, ssize_t *len, size_t bufsiz);
 void crlftolf(char **in, ssize_t *len, size_t bufsiz);
@@ -136,11 +137,7 @@ int main(int argc, const char *argv[]) {
 	 socat_opts.bufsiz = strtoul(a, (char **)&a, 0);
 	 break;
       case 's':
-	 {
-	    char l[2];
-	    l[0] = '0'+E_FATAL; l[1] = '\0';
-	    diag_set('e', l); break;
-	 }
+	 diag_set_int('e', E_FATAL); break;
       case 't': if (arg1[0][2]) {
 	    a = *arg1+2;
 	 } else {
@@ -408,6 +405,7 @@ int closing = 0;	/* 0..no eof yet, 1..first eof just occurred,
 /* call this function when the common command line options are parsed, and the
    addresses are extracted (but not resolved). */
 int socat(const char *address1, const char *address2) {
+   int mayexec;
 
 #if 1
    if (Signal(SIGPIPE, SIG_IGN) == SIG_ERR) {
@@ -416,17 +414,20 @@ int socat(const char *address1, const char *address2) {
 #endif
 
    if (socat_opts.lefttoright) {
-      if ((sock1 = xioopen(address1, O_RDONLY)) == NULL) {
+      if ((sock1 = xioopen(address1, XIO_RDONLY|XIO_MAYFORK|XIO_MAYCHILD|XIO_MAYCONVERT)) == NULL) {
 	 return -1;
       }
+      xiosetsigchild(sock1, socat_sigchild);
    } else if (socat_opts.righttoleft) {
-      if ((sock1 = xioopen(address1, O_WRONLY)) == NULL) {
+      if ((sock1 = xioopen(address1, XIO_WRONLY|XIO_MAYFORK|XIO_MAYCHILD|XIO_MAYCONVERT)) == NULL) {
 	 return -1;
       }
+      xiosetsigchild(sock1, socat_sigchild);
    } else {
-      if ((sock1 = xioopen(address1, O_RDWR)) == NULL) {
+      if ((sock1 = xioopen(address1, XIO_RDWR|XIO_MAYFORK|XIO_MAYCHILD|XIO_MAYCONVERT)) == NULL) {
 	 return -1;
       }
+      xiosetsigchild(sock1, socat_sigchild);
    }
 #if 1	/*! */
    if (XIO_READABLE(sock1) &&
@@ -460,20 +461,24 @@ int socat(const char *address1, const char *address2) {
    }
 #endif
 
+   mayexec = (sock1->common.flags&XIO_DOESCONVERT ? 0 : XIO_MAYEXEC);
    if (XIO_WRITABLE(sock1)) {
       if (XIO_READABLE(sock1)) {
-	 if ((sock2 = xioopen(address2, O_RDWR)) == NULL) {
+	 if ((sock2 = xioopen(address2, XIO_RDWR|XIO_MAYFORK|XIO_MAYCHILD|mayexec|XIO_MAYCONVERT)) == NULL) {
 	    return -1;
 	 }
+	 xiosetsigchild(sock2, socat_sigchild);
       } else {
-	 if ((sock2 = xioopen(address2, O_RDONLY)) == NULL) {
+	 if ((sock2 = xioopen(address2, XIO_RDONLY|XIO_MAYFORK|XIO_MAYCHILD|mayexec|XIO_MAYCONVERT)) == NULL) {
 	    return -1;
 	 }
+	 xiosetsigchild(sock2, socat_sigchild);
       }
    } else {	/* assuming sock1 is readable */
-      if ((sock2 = xioopen(address2, O_WRONLY)) == NULL) {
+      if ((sock2 = xioopen(address2, XIO_WRONLY|XIO_MAYFORK|XIO_MAYCHILD|mayexec|XIO_MAYCONVERT)) == NULL) {
 	 return -1;
       }
+      xiosetsigchild(sock2, socat_sigchild);
    }
 #if 1	/*! */
    if (XIO_READABLE(sock2) &&
@@ -582,12 +587,11 @@ int _socat(void) {
    if (socat_opts.debug) {
       int fdi, fdo;
       int msglevel, exitlevel;
-      char diagtxt[2];
 
       msglevel = diag_get_int('D');	/* save current message level */
-      diag_set('D', "4");	/* only print errors and fatals in filan */
+      diag_set_int('D', E_ERROR);	/* only print errors and fatals in filan */
       exitlevel = diag_get_int('e');	/* save current exit level */
-      diag_set('e', "5");	/* only exit on fatals */
+      diag_set_int('e', E_FATAL);	/* only exit on fatals */
 
       fdi = XIO_GETRDFD(sock1);
       fdo = XIO_GETWRFD(sock1);
@@ -603,10 +607,8 @@ int _socat(void) {
 	 filan_fd(fdo, stderr);
       }
 
-      diagtxt[0] = exitlevel+'0'; diagtxt[1] = '\0';
-      diag_set('e', diagtxt);	/* restore old exit level */
-      diagtxt[0] = msglevel+'0'; diagtxt[1] = '\0';
-      diag_set('D', diagtxt);	/* restore old message level */
+      diag_set_int('e', exitlevel);	/* restore old exit level */
+      diag_set_int('D', msglevel);	/* restore old message level */
    }
 #endif /* WITH_FILAN */
 
@@ -625,7 +627,7 @@ int _socat(void) {
       diag_set('y', xioopts.syslogfac);
       xiosetopt('l', "\0");
    }
-   Notice4("starting data transfer loop with [%d,%d] and [%d,%d]",
+   Notice4("starting data transfer loop with FDs [%d,%d] and [%d,%d]",
 	   XIO_GETRDFD(sock1), XIO_GETWRFD(sock1),
 	   XIO_GETRDFD(sock2), XIO_GETWRFD(sock2));
    while (XIO_RDSTREAM(sock1)->eof <= 1 ||
@@ -1003,8 +1005,8 @@ int cv_newline(unsigned char **buff, ssize_t *bytes,
    return 0;
 }
 
-void socat_signal(int sig) {
-   switch (sig) {
+void socat_signal(int signum) {
+   switch (signum) {
    case SIGQUIT:
    case SIGILL:
    case SIGABRT:
@@ -1012,12 +1014,25 @@ void socat_signal(int sig) {
    case SIGFPE:
    case SIGSEGV:
    case SIGPIPE:
-      Error1("exiting on signal %d", sig); break;
+      diag_set_int('x', 128+signum);	/* in case Error exits for us */
+      Error1("exiting on signal %d", signum);
+      diag_set_int('x', 0);	/* in case Error did not exit */
+      break;
    case SIGTERM:
-      Warn1("exiting on signal %d", sig); break;
+      Warn1("exiting on signal %d", signum); break;
    case SIGHUP:  
    case SIGINT:
-      Notice1("exiting on signal %d", sig); break;
+      Notice1("exiting on signal %d", signum); break;
    }
-   Exit(1);
+   Exit(128+signum);
+}
+
+/* this is the callback when the child of an address died */
+static int socat_sigchild(struct single *file) {
+   if (file->ignoreeof && !closing) {
+      sleepon = 1;
+   } else {
+      file->eof = 1; closing = 1;
+   }
+   return 0;
 }
