@@ -1,5 +1,5 @@
-/* $Id: sycls.c,v 1.64 2004/10/24 13:30:25 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2004 */
+/* $Id: sycls.c,v 1.65 2005/03/12 10:55:38 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* explicit system call and C library trace function, for those who miss strace
@@ -543,7 +543,7 @@ int Fcntl_lock(int fd, int cmd, struct flock *l) {
 
 int Ftruncate(int fd, off_t length) {
    int retval, _errno;
-   Debug2("ftruncate(%d, %d)", fd, length);
+   Debug2("ftruncate(%d, "F_off")", fd, length);
    retval = ftruncate(fd, length);
    _errno = errno;
    Debug1("ftruncate() -> %d", retval);
@@ -552,9 +552,9 @@ int Ftruncate(int fd, off_t length) {
 }
 
 #if HAVE_FTRUNCATE64
-int Ftruncate64(int fd, off_t length) {
+int Ftruncate64(int fd, off64_t length) {
    int retval, _errno;
-   Debug2("ftruncate64(%d, %d)", fd, length);
+   Debug2("ftruncate64(%d, "F_off64")", fd, length);
    retval = ftruncate64(fd, length);
    _errno = errno;
    Debug1("ftruncate64() -> %d", retval);
