@@ -1,12 +1,14 @@
 #! /bin/bash
-# $Id: test.sh,v 1.90 2004/11/18 20:25:00 gerhard Exp $
-# Copyright Gerhard Rieger 2001-2004
+# $Id: test.sh,v 1.95 2005/03/18 21:22:25 gerhard Exp $
+# Copyright Gerhard Rieger 2001-2005
 # Published under the GNU General Public License V.2, see file COPYING
 
 # perform lots of tests on socat
 
 # this script uses functions; you need a shell that supports them
 
+# you can pass general options to socat via $OPTS
+
 #set -vx
 
 withroot=0
@@ -77,19 +79,19 @@ ECHO="echo $E"
 PRINTF="printf"
 
 case "$TERM" in
-vt100|xterm)
+vt100|vt320|linux|xterm|dtterm|aixterm)
 	# there are different behaviours of printf (and echo)
 	# on some systems, echo behaves different than printf...
 	if [ $($PRINTF "\0101") = "A" ]; then
-		RED="\023331m"
-		GREEN="\023332m"
-		YELLOW="\023333m"
-		NORMAL="\023339m"
+		RED="\0033[31m"
+		GREEN="\0033[32m"
+		YELLOW="\0033[33m"
+		NORMAL="\0033[39m"
 	else
-		RED="\23331m"
-		GREEN="\23332m"
-		YELLOW="\23333m"
-		NORMAL="\23339m"
+		RED="\033[31m"
+		GREEN="\033[32m"
+		YELLOW="\033[33m"
+		NORMAL="\033[39m"
 	fi
 	OK="${GREEN}OK${NORMAL}"
 	FAILED="${RED}FAILED${NORMAL}"
@@ -1248,6 +1250,8 @@ waittcp6port () {
     while [ $timeout -gt 0 ]; do
 	case "$UNAME" in
 	Linux)   l=$(netstat -an |grep '^tcp .* [0-9a-f:]*:'$port' .* LISTEN') ;;
+	FreeBSD) l=$(netstat -an |grep -i 'tcp[46][6 ] .*[0-9*][:.]'$port' .* listen') ;;
+	OpenBSD) l=$(netstat -an |grep -i 'tcp6 .*[0-9*][:.]'$port' .* listen') ;;
 	AIX)	 l=$(netstat -an |grep '^tcp[6 ]       0      0 .*[*0-9]\.'$port' .* LISTEN$') ;;
 	#SunOS)   l=$(netstat -an -f inet6 |grep '.*[1-9*]\.'$port' .*\*                0 .* LISTEN') ;;
 	#OSF1)    l=$(/usr/sbin/netstat -an |grep '^tcp6       0      0  .*[0-9*]\.'$port' [ ]*\*\.\* [ ]*LISTEN') /*?*/;;
@@ -1274,7 +1278,8 @@ waitudp6port () {
     while [ $timeout -gt 0 ]; do
 	case "$UNAME" in
 	Linux)   l=$(netstat -an |grep '^udp .* .*[0-9*:]:'$port' [ ]*:::\*') ;;
-	#FreeBSD) l=$(netstat -an |grep '^udp*[6] .*[0-9*]\.'$port' .* \*\.\*') ;;
+	FreeBSD) l=$(netstat -an |grep '^udp46 .*[0-9*]\.'$port' .* \*\.\*') ;;
+    	OpenBSD) l=$(netstat -an |grep '^udp6 .*[0-9*]\.'$port' [ ]* \*\.\*') ;;
 	#Darwin)  l=$(netstat -an |grep '^udp6.* .*[0-9*]\.'$port' .* \*\.\* ') ;;
 	AIX)	 l=$(netstat -an |grep '^udp[6 ]       0      0 .*[*0-9]\.'$port' .* \*\.\*[ ]*$') ;;
 	#SunOS)   l=$(netstat -an -f inet6 |grep '.*[1-9*]\.'$port' [ ]*Idle') ;;
@@ -1319,6 +1324,16 @@ gentestcert () {
     cat $name.key $name.crt >$name.pem
 }
 
+# generate a test DSA key and certificate
+gentestdsacert () {
+    local name="$1"
+    if [ -f $name.key -a -f $name.crt -a -f $name.pem ]; then return; fi
+    openssl dsaparam -out $name-dsa.pem 512 >/dev/null 2>&1
+    openssl dhparam -dsaparam -out $name-dh.pem 512 >/dev/null 2>&1
+    openssl req -newkey dsa:$name-dsa.pem -keyout $name.key -nodes -x509 -config testcert.conf -out $name.crt >/dev/null 2>&1
+    cat $name-dsa.pem $name-dh.pem $name.key $name.crt >$name.pem
+}
+
 TESTS="%$(echo "$TESTS" |tr ' ' '%')%"
 
 NAME=UNISTDIO
@@ -3069,9 +3084,9 @@ te="$td/test$N.stderr"
 printf "test %2d $TEST... " $N
 type=$($FILAN -f . 2>$te |tail -1 |awk '{print($2);}')
 if [ "$type" = "dir" ]; then
-    printf "$OK\n"
+    $PRINTF "$OK\n"
 else
-    printf "$FAILED\n"
+    $PRINTF "$FAILED\n"
     cat "$te"
 fi
 esac
@@ -3091,9 +3106,9 @@ spid=$!
 waitfile "$ts" 1
 type=$($FILAN -f "$ts" 2>$te2 |tail -1 |awk '{print($2);}')
 if [ "$type" = "socket" ]; then
-    printf "$OK\n"
+    $PRINTF "$OK\n"
 else
-    printf "$FAILED\n"
+    $PRINTF "$FAILED\n"
     cat "$te1"
     cat "$te2"
 fi
@@ -3136,9 +3151,9 @@ $SOCAT $opts -lpsocat4 unix:"$ts" echo 2>"$te4"
 wait
 #
 if echo "$da" |diff - "$tf"> "$tdiff"; then
-    printf "$OK\n"
+    $PRINTF "$OK\n"
 else
-    printf "${YELLOW}FAILED${NORMAL}\n"
+    $PRINTF "${YELLOW}FAILED${NORMAL}\n"
     cat "$tdiff"
     cat "$te1"
     #cat "$te2"	# not of interest
@@ -3177,9 +3192,125 @@ fi
 esac
 N=$((N+1))
 
+
+NAME=CONNECTTIMEOUT
+case "$TESTS" in
+*%$NAME%*|*%FUNCTIONS%*)
+TEST="$NAME: test the connect-timeout option"
+if ! feat=$(testaddrs tcp); then
+    $PRINTF "test %2d $TEST... ${YELLOW}$(echo $feat| tr 'a-z' 'A-Z') not available${NORMAL}\n" $N
+elif ! feat=$(testoptions connect-timeout); then
+    $PRINTF "test %2d $TEST... ${YELLOW}$(echo $feat| tr 'a-z' 'A-Z') not available${NORMAL}\n" $N
+else
+# we need a hanging connection attempt, guess an address for this
+case "$UNAME" in
+Linux) HANGIP=1.0.0.1 ;;
+*) HANGIP=255.255.255.254 ;;
+esac
+te1="$td/test$N.stderr1"
+tk1="$td/test$N.kill1"
+te2="$td/test$N.stderr2"
+tk2="$td/test$N.kill2"
+$PRINTF "test %2d $TEST... " $N
+# first, try to make socat hang and see if it can be killed
+$SOCAT $opts - tcp:$HANGIP:1 >"$te1" 2>&1 </dev/null &
+pid1=$!
+sleep 2
+if ! kill $pid1 2>"$tk1"; then
+    $PRINTF "${YELLOW}does not hang${NORMAL}\n"
+else
+# second, set connect-timeout and see if socat exits before kill
+$SOCAT $opts - tcp:$HANGIP:1,connect-timeout=1.0 >"$te2" 2>&1 </dev/null &
+pid2=$!
+sleep 2
+if kill $pid2 2>"$tk2"; then
+    $PRINTF "$FAILED\n"
+else
+    $PRINTF "$OK\n"
+fi
+fi
+fi
+esac
+N=$((N+1))
+
+
+NAME=OPENSSLLISTENDSA
+case "$TESTS" in
+*%$NAME%*|*%FUNCTIONS%*)
+TEST="$NAME: openssl listen with DSA certificate"
+if ! testaddrs openssl >/dev/null; then
+    $PRINTF "test %2d $TEST... ${YELLOW}OPENSSL not available${NORMAL}\n" $N
+else
+SRVCERT=testsrvdsa
+gentestdsacert $SRVCERT
+tf="$td/test$N.stdout"
+te="$td/test$N.stderr"
+tdiff="$td/test$N.diff"
+da=$(date)
+CMD2="$SOCAT $opts OPENSSL-LISTEN:$PORT,reuseaddr,$SOCAT_EGD,cert=$SRVCERT.pem,key=$SRVCERT.key pipe"
+CMD="$SOCAT $opts - openssl:$LOCALHOST:$PORT,verify=0,$SOCAT_EGD"
+$PRINTF "test %2d $TEST... " $N
+eval "$CMD2 2>${te}1 &"
+pid=$!	# background process id
+waittcp4port $PORT
+echo "$da" |$CMD >$tf 2>"${te}2"
+if ! echo "$da" |diff - "$tf" >"$tdiff"; then
+    $PRINTF "$FAILED\n"
+    echo "$CMD2 &"
+    echo "$CMD"
+    cat "${te}1"
+    cat "${te}2"
+    cat "$tdiff"
+else
+   $PRINTF "$OK\n"
+   if [ -n "$debug" ]; then cat ${te}1 ${te}2; fi
+fi
+kill $pid 2>/dev/null
+fi
+esac
+PORT=$((PORT+1))
+N=$((N+1))
+
+
+signum() {
+    signam="$1"
+    kill -l </dev/null |
+    while read l; do printf "%s %s\n%s %s\n%s %s\n%s %s\n" $l; done |
+    grep -e "SIG$signam\$" |
+    cut -d ')' -f 1
+}
+
+# problems with QUIT, INT (are blocked in system() )
+for signam in TERM ILL; do
+NAME=EXITCODESIG$signam
+case "$TESTS" in
+*%$NAME%*|*%FUNCTIONS%*)
+TEST="$NAME: exit status when dying on SIG$signam"
+SIG=$(signum $signam)
+te="$td/test$N.stderr"
+tpp="$td/test$N.ppid"
+tp="$td/test$N.pid"
+$PRINTF "test %2d $TEST... " $N
+(sleep 1; kill -$SIG $(cat "$tpp")) &
+# a simple "system:echo $PPID..." does not work on NetBSD, OpenBSD
+$SOCAT $opts echo system:'exec /bin/bash -c "echo \$PPID '">$tpp"'; echo \$$ '">$tp; read x\"",nofork 2>"$te"; stat=$?
+sleep 1; kill $(cat $tp)
+wait
+if [ "$stat" -eq $((128+$SIG)) ]; then
+    $PRINTF "$OK\n"
+else
+    $PRINTF "$FAILED\n"
+    cat "$te"
+fi
+;;
+esac
+N=$((N+1))
+done
+
+
 #==============================================================================
 
-rm -f testsrv.* testcli.*
+rm -f testsrv.* testcli.* testsrvdsa*
 
 # end
 
@@ -3197,9 +3328,9 @@ TEST="$NAME: !!!"
 printf "test %2d $TEST... " $N
 !!!
 if [ !!! ]; then
-    printf "$OK\n"
+    $PRINTF "$OK\n"
 else
-    printf "$FAILED\n"
+    $PRINTF "$FAILED\n"
     cat "$te"
 fi
 esac
