@@ -1,5 +1,5 @@
-/* $Id: xio-listen.c,v 1.30 2004/06/20 21:30:33 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2004 */
+/* $Id: xio-listen.c,v 1.32 2005/03/18 20:23:18 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for listen socket options */
@@ -45,7 +45,7 @@ int allow_severity=10, deny_severity=10;
    loop in the calling function.
    With fork, the forever/retry of the child process are set to 0
  */
-int _xioopen_listen(struct single *xfd, struct sockaddr *us, socklen_t uslen,
+int _xioopen_listen(struct single *xfd, int xioflags, struct sockaddr *us, socklen_t uslen,
 		 struct opt *opts, int pf, int socktype, int proto, int level) {
    struct sockaddr sa;
    socklen_t salen;
@@ -70,6 +70,14 @@ int _xioopen_listen(struct single *xfd, struct sockaddr *us, socklen_t uslen,
 
    retropt_bool(opts, OPT_FORK, &dofork);
 
+   if (dofork) {
+      if (!(xioflags & XIO_MAYFORK)) {
+	 Error("option fork not allowed here");
+	 return STAT_NORETRY;
+      }
+      xfd->flags |= XIO_DOESFORK;
+   }
+
    applyopts_single(xfd, opts, PH_INIT);
 
 #if 1
