@@ -1,5 +1,5 @@
-/* $Id: xio-listen.h,v 1.7 2004/06/06 17:33:22 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2004 */
+/* $Id: xio-listen.h,v 1.8 2005/03/13 12:15:59 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xio_listen_h_included
@@ -10,7 +10,8 @@ extern const struct optdesc opt_fork;
 extern const struct optdesc opt_range;
 extern const struct optdesc opt_tcpwrappers;
 
-int _xioopen_listen(struct single *fd, struct sockaddr *us, socklen_t uslen,
+int _xioopen_listen(struct single *fd, int xioflags,
+		    struct sockaddr *us, socklen_t uslen,
 		 struct opt *opts, int pf, int socktype, int proto, int level);
 
 #endif /* !defined(__xio_listen_h_included) */
