@@ -1,5 +1,5 @@
-/* $Id: xio-openssl.c,v 1.13 2004/06/20 21:30:37 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2002-2004 */
+/* $Id: xio-openssl.c,v 1.17 2005/03/18 20:23:45 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2002-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the implementation of the openssl addresses */
@@ -36,11 +36,11 @@
  */
 
 /* static declaration of ssl's open function */
-static int xioopen_openssl_connect(char *arg, int rw, xiofile_t *fd, unsigned groups,
+static int xioopen_openssl_connect(char *arg, int xioflags, xiofile_t *fd, unsigned groups,
 			   int dummy1, int dummy2, void *dummyp1);
 
 /* static declaration of ssl's open function */
-static int xioopen_openssl_listen(char *arg, int rw, xiofile_t *fd, unsigned groups,
+static int xioopen_openssl_listen(char *arg, int xioflags, xiofile_t *fd, unsigned groups,
 			   int dummy1, int dummy2, void *dummyp1);
 static int openssl_SSL_ERROR_SSL(int level, const char *funcname);
 static int openssl_handle_peer_certificate(struct single *xfd, bool opt_ver,
@@ -97,6 +97,7 @@ const struct optdesc opt_openssl_method     = { "openssl-method",     "method",
 const struct optdesc opt_openssl_verify     = { "openssl-verify",     "verify",  OPT_OPENSSL_VERIFY,     GROUP_OPENSSL, PH_SPEC, TYPE_BOOL,   OFUNC_SPEC };
 const struct optdesc opt_openssl_certificate = { "openssl-certificate", "cert",  OPT_OPENSSL_CERTIFICATE, GROUP_OPENSSL, PH_SPEC, TYPE_FILENAME, OFUNC_SPEC };
 const struct optdesc opt_openssl_key         = { "openssl-key",         "key",   OPT_OPENSSL_KEY,         GROUP_OPENSSL, PH_SPEC, TYPE_FILENAME, OFUNC_SPEC };
+const struct optdesc opt_openssl_dhparam     = { "openssl-dhparam",     "dh",    OPT_OPENSSL_DHPARAM,     GROUP_OPENSSL, PH_SPEC, TYPE_FILENAME, OFUNC_SPEC };
 const struct optdesc opt_openssl_cafile      = { "openssl-cafile",     "cafile", OPT_OPENSSL_CAFILE,      GROUP_OPENSSL, PH_SPEC, TYPE_FILENAME, OFUNC_SPEC };
 const struct optdesc opt_openssl_capath      = { "openssl-capath",     "capath", OPT_OPENSSL_CAPATH,      GROUP_OPENSSL, PH_SPEC, TYPE_FILENAME, OFUNC_SPEC };
 const struct optdesc opt_openssl_egd         = { "openssl-egd",        "egd",    OPT_OPENSSL_EGD,         GROUP_OPENSSL, PH_SPEC, TYPE_FILENAME, OFUNC_SPEC };
@@ -106,7 +107,7 @@ const struct optdesc opt_openssl_pseudo      = { "openssl-pseudo",     "pseudo",
 /* the open function for OpenSSL client */
 static int
    xioopen_openssl_connect(char *arg,	/* the arguments in the address string */
-		   int rw,	/* is the open meant for reading (0),
+		   int xioflags,	/* is the open meant for reading (0),
 				   writing (1), or both (2) ? */
 		   xiofile_t *xxfd,	/* a xio file descriptor structure,
 				   already allocated */
@@ -136,6 +137,12 @@ static int
    char *opt_cert = NULL;	/* file name of client certificate */
    int result;
 
+   if (!(xioflags & XIO_MAYCONVERT)) {
+      Error("address with data processing not allowed here");
+      return STAT_NORETRY;
+   }
+   xfd->flags |= XIO_DOESCONVERT;
+
    if (portname = strchr(arg, ':')) {
       *portname++ = '\0';
    } else {
@@ -325,7 +332,7 @@ int _xioopen_openssl_connect(struct single *xfd,
 
 static int
    xioopen_openssl_listen(char *arg,	/* the arguments in the address string */
-		   int rw,	/* is the open meant for reading (0),
+		   int xioflags,	/* is the open meant for reading (0),
 				   writing (1), or both (2) ? */
 		   xiofile_t *xxfd,	/* a xio file descriptor structure,
 				   already allocated */
@@ -349,6 +356,12 @@ static int
    char *opt_cert = NULL;	/* file name of server certificate */
    int result;
 
+   if (!(xioflags & XIO_MAYCONVERT)) {
+      Error("address with data processing not allowed here");
+      return STAT_NORETRY;
+   }
+   xfd->flags |= XIO_DOESCONVERT;
+
    if (a2 = strchr(arg, ',')) {
       *a2++ = '\0';
    }
@@ -388,7 +401,8 @@ static int
 
       /* tcp listen; this can fork() for us; it only returns on error or on
 	 successful establishment of tcp connection */
-      result = _xioopen_listen(xfd, (struct sockaddr *)us, sizeof(*us),
+      result = _xioopen_listen(xfd, xioflags,
+			       (struct sockaddr *)us, sizeof(*us),
 			       opts, PF_INET, socktype, IPPROTO_TCP,
 #if WITH_RETRY
 			       (xfd->retry||xfd->forever)?E_INFO:E_ERROR
@@ -557,6 +571,7 @@ int
    char *me_str = NULL;	/* method string */
    char *ci_str = NULL;	/* cipher string */
    char *opt_key  = NULL;	/* file name of client private key */
+   char *opt_dhparam = NULL;	/* file name of DH params */
    char *opt_cafile = NULL;	/* certificate authority file */
    char *opt_capath = NULL;	/* certificate authority directory */
    char *opt_egd = NULL;	/* entropy gathering daemon socket path */
@@ -573,9 +588,13 @@ int
    retropt_string(opts, OPT_OPENSSL_CAFILE, &opt_cafile);
    retropt_string(opts, OPT_OPENSSL_CAPATH, &opt_capath);
    retropt_string(opts, OPT_OPENSSL_KEY, &opt_key);
+   retropt_string(opts, OPT_OPENSSL_DHPARAM, &opt_dhparam);
    retropt_string(opts, OPT_OPENSSL_EGD, &opt_egd);
    retropt_bool(opts,OPT_OPENSSL_PSEUDO, &opt_pseudo);
 
+   OpenSSL_add_all_algorithms();
+   OpenSSL_add_all_ciphers();
+   OpenSSL_add_all_digests();
    sycSSL_load_error_strings();
 
    /* OpenSSL preparation */
@@ -671,6 +690,9 @@ int
    }
 
    if (opt_cert) {
+      BIO *bio;
+      DH *dh;
+
       if (sycSSL_CTX_use_certificate_chain_file(*ctx, opt_cert) <= 0) {
 	 /*! trace functions */
 	 /*0 ERR_print_errors_fp(stderr);*/
@@ -683,14 +705,29 @@ int
 	 }
 	 return STAT_RETRYLATER;
       }
-   }
 
-   if (opt_cert) {
       if (sycSSL_CTX_use_PrivateKey_file(*ctx, opt_key?opt_key:opt_cert, SSL_FILETYPE_PEM) <= 0) {
 	 /*ERR_print_errors_fp(stderr);*/
 	 openssl_SSL_ERROR_SSL(E_ERROR/*!*/, "SSL_CTX_use_PrivateKey_file");
 	 return STAT_RETRYLATER;
       }
+
+      if (opt_dhparam == NULL) {
+	 opt_dhparam = (char *)opt_cert;
+      }
+      if ((bio = sycBIO_new_file(opt_dhparam, "r")) == NULL) {
+	 Warn2("BIO_new_file(\"%s\", \"r\"): %s",
+	       opt_dhparam, strerror(errno));
+      } else {
+	 if ((dh = sycPEM_read_bio_DHparams(bio, NULL, NULL, NULL)) == NULL) {
+	    Info1("PEM_read_bio_DHparams(%p, NULL, NULL, NULL): error", bio);
+	 } else {
+	    BIO_free(bio);
+	    if (sycSSL_CTX_set_tmp_dh(*ctx, dh) == 0) {
+	       Error2("SSL_CTX_set_tmp_dh(%p, %p): error", ctx, dh);
+	    }
+	 }
+      }
    }
 
    /* set pre ssl-connect options */
