@@ -1,5 +1,5 @@
-/* $Id: xio-pipe.c,v 1.16 2004/06/06 17:33:22 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2004 */
+/* $Id: xio-pipe.c,v 1.17 2005/03/13 12:10:01 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for opening addresses of pipe type */
@@ -80,7 +80,8 @@ int xioopen_fifo_unnamed(char *arg, xiofile_t *sock) {
 
 
 /* open a named pipe/fifo */
-static int xioopen_fifo(char *a1, int rw, xiofile_t *fd, unsigned groups, int dummy1, int dummy2, void *dummyp1) {
+static int xioopen_fifo(char *a1, int xioflags, xiofile_t *fd, unsigned groups, int dummy1, int dummy2, void *dummyp1) {
+   int rw = (xioflags & XIO_ACCMODE);
    struct opt *opts = NULL;
 #if HAVE_STAT64
    struct stat64 pipstat;
