@@ -1,5 +1,5 @@
-/* $Id: xio-progcall.c,v 1.43 2004/08/22 06:24:47 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2004 */
+/* $Id: xio-progcall.c,v 1.45 2005/03/18 20:24:20 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains common code dealing with program calls (exec, system) */
@@ -42,7 +42,7 @@ const struct optdesc opt_sigquit = { "sigquit",   NULL, OPT_SIGQUIT,     GROUP_P
    return<0: error occurred, assume parent process and no child exists !!!
  */
 int _xioopen_foxec(char *comma,
-		int rw,	/* O_RDONLY etc. */
+		int xioflags,	/* XIO_RDONLY etc. */
 		struct single *fd,
 		unsigned groups,
 		   struct opt **copts	/* in: opts; out: opts for child */
@@ -50,6 +50,7 @@ int _xioopen_foxec(char *comma,
    struct opt *popts;	/* parent process options */
    int numleft;
    int d, type, protocol, sv[2], rdpip[2], wrpip[2];
+   int rw = (xioflags & XIO_ACCMODE);
    bool usepipes = false;
 #if HAVE_PTY
    int ptyfd = -1, ttyfd = -1;
@@ -102,6 +103,13 @@ int _xioopen_foxec(char *comma,
    retropt_ushort(popts, OPT_FDOUT, (unsigned short *)&fdo);
 
    if (withfork) {
+      if (!(xioflags&XIO_MAYCHILD)) {
+	 Error("cannot fork off child process here");
+	 /*!! free something */
+	 return -1;
+      }
+      fd->flags |= XIO_DOESCHILD;
+
 #if HAVE_PTY
       Notice2("forking off child, using %s for %s",
 	    &("socket\0\0pipes\0\0\0pty\0\0\0\0\0"[(usepipes<<3)|(usepty<<4)]),
@@ -115,15 +123,19 @@ int _xioopen_foxec(char *comma,
    applyopts(-1, popts, PH_PREBIGEN);
 
    if (!withfork) {
-      struct single *stream1, *stream2;
+      /*0 struct single *stream1, *stream2;*/
+
+      if (!(xioflags & XIO_MAYEXEC /* means exec+nofork */)) {
+	 Error("option nofork is not allowed here");
+	 /*!! free something */
+	 return -1;
+      }
+      fd->flags |= XIO_DOESEXEC;
 
       free(*copts);
       *copts = moveopts(popts, GROUP_ALL);
-      /* what if WE are sock1 ? */
-      if (sock1 == NULL) {
-	 Fatal("nofork option must no be applied to first socat address");
-      }
 
+#if 0 /*!! */
       if (sock1->tag == XIO_TAG_DUAL) {
 	 stream1 = &sock1->dual.stream[0]->stream;
 	 stream2 = &sock1->dual.stream[1]->stream;
@@ -141,42 +153,43 @@ int _xioopen_foxec(char *comma,
 	  stream1->ignoreeof || stream2->ignoreeof) {
 	 Warn("due to option nofork, address1 options for lineterm and igoreeof do not apply");
       }
+#endif
 
-      /*! problem: when fdi==WRFD(sock1) or fdo==RDFD(sock1) */
-      if (rw != O_WRONLY) {
-	 if (XIO_GETRDFD(sock1) == fdi) {
+      /*! problem: when fdi==WRFD(sock[0]) or fdo==RDFD(sock[0]) */
+      if (rw != XIO_WRONLY) {
+	 if (XIO_GETRDFD(sock[0]/*!!*/) == fdi) {
 	    if (Fcntl_l(fdi, F_SETFD, 0) < 0) {
 	       Warn2("fcntl(%d, F_SETFD, 0): %s", fdi, strerror(errno));
 	    }
-	    if (Dup2(XIO_GETRDFD(sock1), fdi) < 0) {
+	    if (Dup2(XIO_GETRDFD(sock[0]), fdi) < 0) {
 	       Error3("dup2(%d, %d): %s",
-		      XIO_GETRDFD(sock1), fdi, strerror(errno));
+		      XIO_GETRDFD(sock[0]), fdi, strerror(errno));
 	    }
-	    /*0 Info2("dup2(%d, %d)", XIO_GETRDFD(sock1), fdi);*/
+	    /*0 Info2("dup2(%d, %d)", XIO_GETRDFD(sock[0]), fdi);*/
 	 } else {
-	    if (Dup2(XIO_GETRDFD(sock1), fdi) < 0) {
+	    if (Dup2(XIO_GETRDFD(sock[0]), fdi) < 0) {
 	       Error3("dup2(%d, %d): %s",
-		      XIO_GETRDFD(sock1), fdi, strerror(errno));
+		      XIO_GETRDFD(sock[0]), fdi, strerror(errno));
 	    }
-	    /*0 Info2("dup2(%d, %d)", XIO_GETRDFD(sock1), fdi);*/
+	    /*0 Info2("dup2(%d, %d)", XIO_GETRDFD(sock[0]), fdi);*/
 	 }
       }
-      if (rw != O_RDONLY) {
-	 if (XIO_GETWRFD(sock1) == fdo) {
+      if (rw != XIO_RDONLY) {
+	 if (XIO_GETWRFD(sock[0]) == fdo) {
 	    if (Fcntl_l(fdo, F_SETFD, 0) < 0) {
 	       Warn2("fcntl(%d, F_SETFD, 0): %s", fdo, strerror(errno));
 	    }
-	    if (Dup2(XIO_GETWRFD(sock1), fdo) < 0) {
+	    if (Dup2(XIO_GETWRFD(sock[0]), fdo) < 0) {
 	       Error3("dup2(%d, %d): %s)",
-		      XIO_GETWRFD(sock1), fdo, strerror(errno));
+		      XIO_GETWRFD(sock[0]), fdo, strerror(errno));
 	    }
-	    /*0 Info2("dup2(%d, %d)", XIO_GETWRFD(sock1), fdo);*/
+	    /*0 Info2("dup2(%d, %d)", XIO_GETWRFD(sock[0]), fdo);*/
 	 } else {
-	    if (Dup2(XIO_GETWRFD(sock1), fdo) < 0) {
+	    if (Dup2(XIO_GETWRFD(sock[0]), fdo) < 0) {
 	       Error3("dup2(%d, %d): %s)",
-		      XIO_GETWRFD(sock1), fdo, strerror(errno));
+		      XIO_GETWRFD(sock[0]), fdo, strerror(errno));
 	    }
-	    /*0 Info2("dup2(%d, %d)", XIO_GETWRFD(sock1), fdo);*/
+	    /*0 Info2("dup2(%d, %d)", XIO_GETWRFD(sock[0]), fdo);*/
 	 }
       }
    } else
@@ -281,9 +294,9 @@ int _xioopen_foxec(char *comma,
    if (usepipes) {
       struct opt *popts2, *copts2;
 
-      if (rw == O_RDWR)
+      if (rw == XIO_RDWR)
 	 fd->dtype = DATA_2PIPE;
-      if (rw != O_WRONLY) {
+      if (rw != XIO_WRONLY) {
 	 if (Pipe(rdpip) < 0) {
 	    Error2("pipe(%p): %s", rdpip, strerror(errno));
 	    return STAT_RETRYLATER;
@@ -300,13 +313,13 @@ int _xioopen_foxec(char *comma,
       popts2 = copyopts(popts, GROUP_ALL);
       copts2 = copyopts(*copts, GROUP_ALL);
 
-      if (rw != O_WRONLY) {
+      if (rw != XIO_WRONLY) {
 	 applyopts_cloexec(rdpip[0], popts);
 	 applyopts(rdpip[0], popts, PH_FD);
 	 applyopts(rdpip[1], *copts, PH_FD);
       }
 
-      if (rw != O_RDONLY) {
+      if (rw != XIO_RDONLY) {
 	 if (Pipe(wrpip) < 0) {
 	    Error2("pipe(%p): %s", wrpip, strerror(errno));
 	    return STAT_RETRYLATER;
@@ -315,7 +328,7 @@ int _xioopen_foxec(char *comma,
       /*0 Info2("pipe({%d,%d})", wrpip[0], wrpip[1]);*/
 
       /* wrpip[1]: write by socat; wrpip[0]: read by child */
-      if (rw != O_RDONLY) {
+      if (rw != XIO_RDONLY) {
 	 applyopts_cloexec(wrpip[1], popts2);
 	 applyopts(wrpip[1], popts2, PH_FD);
 	 applyopts(wrpip[0], copts2, PH_FD);
@@ -324,9 +337,9 @@ int _xioopen_foxec(char *comma,
 
       /* this for parent, was after fork */
       switch (rw) {
-      case O_RDONLY: fd->fd = rdpip[0]; break;
-      case O_WRONLY: fd->fd = wrpip[1]; break;
-      case O_RDWR:   fd->fd = rdpip[0];
+      case XIO_RDONLY: fd->fd = rdpip[0]; break;
+      case XIO_WRONLY: fd->fd = wrpip[1]; break;
+      case XIO_RDWR:   fd->fd = rdpip[0];
 	 fd->para.exec.fdout = wrpip[1];
 	 break;
       }
@@ -396,20 +409,20 @@ int _xioopen_foxec(char *comma,
 
 #if HAVE_PTY
 	 if (usepty) {
-	    if (rw != O_RDONLY && fdi != ttyfd) {
+	    if (rw != XIO_RDONLY && fdi != ttyfd) {
 	       if (Dup2(ttyfd, fdi) < 0) {
 		  Error3("dup2(%d, %d): %s", ttyfd, fdi, strerror(errno));
 		  return STAT_RETRYLATER; }
 	       /*0 Info2("dup2(%d, %d)", ttyfd, fdi);*/
 	    }
-	    if (rw != O_WRONLY && fdo != ttyfd) {
+	    if (rw != XIO_WRONLY && fdo != ttyfd) {
 	       if (Dup2(ttyfd, fdo) < 0) {
 		  Error3("dup2(%d, %d): %s", ttyfd, fdo, strerror(errno));
 		  return STAT_RETRYLATER; }
 	       /*0 Info2("dup2(%d, %d)", ttyfd, fdo);*/
 	    }
-	    if ((rw == O_RDONLY || fdi != ttyfd) &&
-		(rw == O_WRONLY || fdo != ttyfd)) {
+	    if ((rw == XIO_RDONLY || fdi != ttyfd) &&
+		(rw == XIO_WRONLY || fdo != ttyfd)) {
 	       applyopts_cloexec(ttyfd, *copts);
 	    }
 
@@ -441,7 +454,7 @@ int _xioopen_foxec(char *comma,
 		  wrpip[0] = tmpo;
 	       }
 	       
-	       if (rw != O_WRONLY && rdpip[1] != fdo) {
+	       if (rw != XIO_WRONLY && rdpip[1] != fdo) {
 		  if (Dup2(rdpip[1], fdo) < 0) {
 		     Error3("dup2(%d, %d): %s", rdpip[1], fdo, strerror(errno));
 		     return STAT_RETRYLATER;
@@ -450,7 +463,7 @@ int _xioopen_foxec(char *comma,
 		  /*0 Info2("dup2(%d, %d)", rdpip[1], fdo);*/
 		  /*0 applyopts_cloexec(fdo, *copts);*/
 	       }
-	       if (rw != O_RDONLY && wrpip[0] != fdi) {
+	       if (rw != XIO_RDONLY && wrpip[0] != fdi) {
 		  if (Dup2(wrpip[0], fdi) < 0) {
 		     Error3("dup2(%d, %d): %s", wrpip[0], fdi, strerror(errno));
 		     return STAT_RETRYLATER;
@@ -467,13 +480,13 @@ int _xioopen_foxec(char *comma,
 	       applyopts(fdo, *copts, PH_LATE2);
 
 	    } else {	/* socketpair */
-	       if (rw != O_RDONLY && fdi != sv[1]) {
+	       if (rw != XIO_RDONLY && fdi != sv[1]) {
 		  if (Dup2(sv[1], fdi) < 0) {
 		     Error3("dup2(%d, %d): %s", sv[1], fdi, strerror(errno));
 		     return STAT_RETRYLATER; }
 		  /*0 Info2("dup2(%d, %d)", sv[1], fdi);*/
 	       }
-	       if (rw != O_WRONLY && fdo != sv[1]) {
+	       if (rw != XIO_WRONLY && fdo != sv[1]) {
 		  if (Dup2(sv[1], fdo) < 0) {
 		     Error3("dup2(%d, %d): %s", sv[1], fdo, strerror(errno));
 		     return STAT_RETRYLATER; }
