@@ -1,5 +1,5 @@
-/* $Id: xio-proxy.c,v 1.17 2004/11/20 08:20:37 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2002-2004 */
+/* $Id: xio-proxy.c,v 1.18 2005/03/13 12:16:32 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2002-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for opening addresses of HTTP proxy CONNECT
@@ -18,7 +18,7 @@
 
 #define PROXYPORT "8080"
 
-static int xioopen_proxy_connect(char *a1, int rw, xiofile_t *fd,
+static int xioopen_proxy_connect(char *a1, int xioflags, xiofile_t *fd,
 				 unsigned groups, int dummy1, int dummy2,
 				 void *dummyp1);
 
@@ -73,7 +73,7 @@ static ssize_t
 #define BUFLEN 2048
 
 
-static int xioopen_proxy_connect(char *a1, int rw, xiofile_t *xxfd,
+static int xioopen_proxy_connect(char *a1, int xioflags, xiofile_t *xxfd,
 				 unsigned groups, int dummy1, int dummy2,
 				 void *dummyp1) {
       /* we expect the form: host:host:port */
