@@ -1,5 +1,5 @@
-/* $Id: xio-pty.c,v 1.11 2004/12/11 13:33:23 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2002-2004 */
+/* $Id: xio-pty.c,v 1.13 2005/03/18 20:24:39 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2002-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for creating pty addresses */
@@ -18,7 +18,7 @@
 
 #define MAXPTYNAMELEN 64
 
-static int xioopen_pty(char *a1, int rw, xiofile_t *fd, unsigned groups, int dummy1, int dummy2, void *dummyp1);
+static int xioopen_pty(char *a1, int xioflags, xiofile_t *fd, unsigned groups, int dummy1, int dummy2, void *dummyp1);
 
 const struct addrdesc addr_pty = { "pty",   3, xioopen_pty, GROUP_NAMED|GROUP_FD|GROUP_TERMIOS|GROUP_PTY, 0, 0, NULL HELP("") };
 
@@ -28,7 +28,7 @@ const struct optdesc opt_pty_wait_slave = { "pty-wait-slave", "wait-slave", OPT_
 const struct optdesc opt_pty_intervall  = { "pty-intervall",  NULL,         OPT_PTY_INTERVALL,  GROUP_PTY, PH_EARLY, TYPE_TIMESPEC, OFUNC_SPEC, 0, 0 };
 #endif /* HAVE_POLL */
 
-static int xioopen_pty(char *a1, int rw, xiofile_t *xfd, unsigned groups, int dummy1, int dummy2, void *dummyp1) {
+static int xioopen_pty(char *a1, int xioflags, xiofile_t *xfd, unsigned groups, int dummy1, int dummy2, void *dummyp1) {
    /* we expect the form: filename */
    struct opt *opts = NULL;
    int ptyfd = -1, ttyfd = -1;
@@ -173,9 +173,9 @@ static int xioopen_pty(char *a1, int rw, xiofile_t *xfd, unsigned groups, int du
       }
 #endif
 #if HAVE_OPENPTY
-   if (useopenpty) {
-      Close(ttyfd);
-   }
+      if (useopenpty) {
+	 Close(ttyfd);
+      }
 #endif /* HAVE_OPENPTY */
 
       /* now we poll until the HUP vanishes - this indicates a slave conn. */
