@@ -1,5 +1,5 @@
-/* $Id: xio-rawip.c,v 1.14 2004/06/20 21:28:12 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2004 */
+/* $Id: xio-rawip.c,v 1.15 2005/03/13 12:16:54 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for opening addresses of raw IP type */
@@ -25,7 +25,7 @@ const struct addrdesc addr_rawip6 = { "ip6",     3, xioopen_rawip, GROUP_FD|GROU
 /* we expect the form: host:protocol */
 /* struct sockaddr_in sa;*/
 /* socklen_t salen;*/
-int xioopen_rawip(char *a1, int rw, xiofile_t *fd, unsigned groups, int pf, int dummy2, void *dummyp1) {
+int xioopen_rawip(char *a1, int xioflags, xiofile_t *fd, unsigned groups, int pf, int dummy2, void *dummyp1) {
    char *a2, *a3;
    struct opt *opts = NULL;
    union sockaddr_union us;
