@@ -1,5 +1,5 @@
-/* $Id: xio-rawip.h,v 1.6 2001/11/04 17:19:20 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001 */
+/* $Id: xio-rawip.h,v 1.7 2005/03/13 12:17:03 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xio_rawip_h_included
@@ -8,6 +8,6 @@
 extern const struct addrdesc addr_rawip4;
 extern const struct addrdesc addr_rawip6;
 
-extern int xioopen_rawip(char *a1, int rw, xiofile_t *fd, unsigned groups, int pf, int dummy2, void *dummyp1);
+extern int xioopen_rawip(char *a1, int xioflags, xiofile_t *fd, unsigned groups, int pf, int dummy2, void *dummyp1);
 
 #endif /* !defined(__xio_rawip_h_included) */
