@@ -1,5 +1,5 @@
-/* $Id: xio-socket.c,v 1.27 2004/12/03 21:22:33 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2004 */
+/* $Id: xio-socket.c,v 1.29 2005/03/18 20:25:29 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for socket related functions */
@@ -116,7 +116,7 @@ const struct optdesc opt_fiosetown   = { "fiosetown", NULL, OPT_FIOSETOWN,   GRO
 const struct optdesc opt_siocspgrp   = { "siocspgrp", NULL, OPT_SIOCSPGRP,   GROUP_SOCKET, PH_PASTSOCKET, TYPE_INT,  OFUNC_IOCTL,  SIOCSPGRP };
 #endif
 const struct optdesc opt_bind        = { "bind",      NULL, OPT_BIND,        GROUP_SOCKET, PH_BIND, TYPE_STRING,OFUNC_SPEC };
-
+const struct optdesc opt_connect_timeout = { "connect-timeout", NULL, OPT_CONNECT_TIMEOUT, GROUP_SOCKET, PH_PASTSOCKET, TYPE_TIMEVAL, OFUNC_OFFSET, (int)&((xiofile_t *)0)->stream.para.socket.connect_timeout };
 
 
 /* a subroutine that is common to all socket addresses that want to connect
@@ -128,6 +128,7 @@ int _xioopen_connect(struct single *xfd, struct sockaddr *us, size_t uslen,
 		     struct sockaddr *them, size_t themlen,
 		     struct opt *opts, int pf, int stype, int proto,
 		     bool alt, int level) {
+   int fcntl_flags = 0;
    char infobuff[256];
    union sockaddr_union la;
    socklen_t lalen = sizeof(la);
@@ -139,6 +140,7 @@ int _xioopen_connect(struct single *xfd, struct sockaddr *us, size_t uslen,
       return STAT_RETRYLATER;	    
    }
 
+   applyopts_offset(xfd, opts);
    applyopts(xfd->fd, opts, PH_PASTSOCKET);
    applyopts(xfd->fd, opts, PH_FD);
 
@@ -239,11 +241,58 @@ int _xioopen_connect(struct single *xfd, struct sockaddr *us, size_t uslen,
 
    applyopts(xfd->fd, opts, PH_CONNECT);
 
+   if (xfd->para.socket.connect_timeout.tv_sec  != 0 ||
+       xfd->para.socket.connect_timeout.tv_usec != 0) {
+      fcntl_flags = Fcntl(xfd->fd, F_GETFL);
+      Fcntl_l(xfd->fd, F_SETFL, fcntl_flags|O_NONBLOCK);
+   }
    if ((result = Connect(xfd->fd, (struct sockaddr *)them, themlen)) < 0) {
       if (errno == EINPROGRESS) {
-	 Warn4("connect(%d, %s, "F_Zd"): %s",
-	       xfd->fd, sockaddr_info(them, infobuff, sizeof(infobuff)),
-	       themlen, strerror(errno));
+	 if (xfd->para.socket.connect_timeout.tv_sec  != 0 ||
+	     xfd->para.socket.connect_timeout.tv_usec != 0) {
+	    struct timeval timeout;
+	    fd_set readfds, writefds, exceptfds;
+	    int result;
+	    Info4("connect(%d, %s, "F_Zd"): %s",
+		  xfd->fd, sockaddr_info(them, infobuff, sizeof(infobuff)),
+		  themlen, strerror(errno));
+	    timeout = xfd->para.socket.connect_timeout;
+	    FD_ZERO(&readfds);  FD_ZERO(&writefds);  FD_ZERO(&exceptfds);
+	    FD_SET(xfd->fd, &readfds);  FD_SET(xfd->fd, &writefds);
+	    result =
+	       Select(xfd->fd+1, &readfds, &writefds, &exceptfds, &timeout);
+	    if (result < 0) {
+	       Msg2(level, "select(%d,,,,): %s", xfd->fd+1, strerror(errno));
+	       return STAT_RETRYLATER;
+	    }
+	    if (result == 0) {
+	       Msg2(level, "connecting to %s: %s",
+		    sockaddr_info(them, infobuff, sizeof(infobuff)),
+		    strerror(ETIMEDOUT));
+	       return STAT_RETRYLATER;
+	    }
+	    if (FD_ISSET(xfd->fd, &readfds)) {
+#if 0
+	       unsigned char dummy[1];
+	       Read(xfd->fd, &dummy, 1);	/* get error message */
+	       Msg2(level, "connecting to %s: %s",
+		    sockaddr_info(them, infobuff, sizeof(infobuff)),
+		    strerror(errno));
+#else
+	       Connect(xfd->fd, them, themlen);	/* get error message */
+	       Msg4(level, "connect(%d, %s, "F_Zd"): %s",
+		     xfd->fd, sockaddr_info(them, infobuff, sizeof(infobuff)),
+		     themlen, strerror(errno));
+#endif
+	       return STAT_RETRYLATER;
+	    }
+	    /* otherwise OK */
+	    Fcntl_l(xfd->fd, F_SETFL, fcntl_flags);
+	 } else {
+	    Warn4("connect(%d, %s, "F_Zd"): %s",
+		  xfd->fd, sockaddr_info(them, infobuff, sizeof(infobuff)),
+		  themlen, strerror(errno));
+	 }
       } else if (errno == EPROTOTYPE) {
 	 /* this is for UNIX domain sockets: a connect attempt seems to be
 	    the only way to distinguish stream and datagram sockets */
