@@ -1,5 +1,5 @@
-/* $Id: xio-socks.c,v 1.22 2004/08/25 15:54:36 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2004 */
+/* $Id: xio-socks.c,v 1.23 2005/03/13 12:17:34 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for opening addresses of socks4 type */
@@ -23,7 +23,7 @@ enum {
 #define SOCKSPORT "1080"
 #define BUFF_LEN (SIZEOF_STRUCT_SOCKS4+512)
 
-static int xioopen_socks4_connect(char *a1, int rw, xiofile_t *fd,
+static int xioopen_socks4_connect(char *a1, int xioflags, xiofile_t *fd,
 				  unsigned groups, int dummy1, int dummy2,
 				  void *dummyp1);
 
@@ -34,7 +34,7 @@ const struct addrdesc addr_socks4_connect = { "socks4", 3, xioopen_socks4_connec
 
 const struct addrdesc addr_socks4a_connect = { "socks4a", 3, xioopen_socks4_connect, GROUP_FD|GROUP_SOCKET|GROUP_SOCK_IP4|GROUP_IP_TCP|GROUP_IP_SOCKS4|GROUP_CHILD|GROUP_RETRY, 1, 0, NULL HELP(":<socks-server>:<host>:<port>") };
 
-static int xioopen_socks4_connect(char *a1, int rw, xiofile_t *xxfd,
+static int xioopen_socks4_connect(char *a1, int xioflags, xiofile_t *xxfd,
 				  unsigned groups, int socks4a, int dummy2,
 				  void *dummyp1) {
    /* we expect the form: host:host:port */
