@@ -1,5 +1,5 @@
-/* $Id: xio-stdio.c,v 1.11 2004/06/06 17:33:22 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2003 */
+/* $Id: xio-stdio.c,v 1.12 2005/03/13 12:17:55 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for opening addresses stdio type */
@@ -13,7 +13,7 @@
 
 #if WITH_STDIO
 
-static int xioopen_stdio(char *arg, int rw, xiofile_t *fd, unsigned groups, int dummy1, int dummy2, void *dummyp1);
+static int xioopen_stdio(char *arg, int xioflags, xiofile_t *fd, unsigned groups, int dummy1, int dummy2, void *dummyp1);
 
 
 const struct addrdesc addr_stdio  = { "stdio",  3, xioopen_stdio, GROUP_FD, 0, 0, NULL HELP(NULL) };
@@ -121,7 +121,8 @@ int xioopen_stdio_bi(char *arg, xiofile_t *sock) {
 
 /* wrap around unidirectional xioopensingle and xioopen_fd to automatically determine stdin or stdout fd depending on rw.
    Do not set FD_CLOEXEC flag. */
-static int xioopen_stdio(char *arg, int rw, xiofile_t *fd, unsigned groups, int dummy1, int dummy2, void *dummyp1) {
+static int xioopen_stdio(char *arg, int xioflags, xiofile_t *fd, unsigned groups, int dummy1, int dummy2, void *dummyp1) {
+   int rw = (xioflags&XIO_ACCMODE);
    Notice2("using %s for %s",
 	   &("stdin\0\0\0stdout\0\0stderr"[rw<<3]),
 	   ddirection[rw]);
