@@ -1,5 +1,5 @@
-/* $Id: xio-tcp.c,v 1.17 2004/12/03 21:53:24 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2004 */
+/* $Id: xio-tcp.c,v 1.18 2005/03/13 12:18:12 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for TCP related functions and options */
@@ -111,7 +111,7 @@ const struct optdesc opt_tcp_tsoptena = { "tcp-tsoptena", "tsoptena", OPT_TCP_TS
 
 #if WITH_IP6 && WITH_LISTEN
 /* we expect the form: port */
-int xioopen_tcp6_listen(char *a1, int rw, xiofile_t *fd,
+int xioopen_tcp6_listen(char *a1, int xioflags, xiofile_t *fd,
 			     unsigned groups, int dummy1, int dummy2,
 			     void *dummyp1) {
    char *a2;
@@ -146,7 +146,7 @@ int xioopen_tcp6_listen(char *a1, int rw, xiofile_t *fd,
    retropt_int(opts, OPT_SO_TYPE, &socktype);
 
    if ((result =
-	_xioopen_listen(&fd->stream,
+	_xioopen_listen(&fd->stream, xioflags,
 			(struct sockaddr *)&us, sizeof(struct sockaddr_in6),
 			opts, PF_INET6, socktype, IPPROTO_TCP,
 #if WITH_RETRY
