@@ -1,5 +1,5 @@
-/* $Id: xio-tcp.h,v 1.7 2002/05/19 08:04:26 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001, 2002 */
+/* $Id: xio-tcp.h,v 1.10 2004/12/03 21:53:24 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2004 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xio_tcp_h_included
@@ -25,6 +25,17 @@ extern const struct optdesc opt_tcp_defer_accept;
 extern const struct optdesc opt_tcp_window_clamp;
 extern const struct optdesc opt_tcp_info;
 extern const struct optdesc opt_tcp_quickack;
+extern const struct optdesc opt_tcp_noopt;
+extern const struct optdesc opt_tcp_nopush;
+extern const struct optdesc opt_tcp_md5sig;
+extern const struct optdesc opt_tcp_sack_disable;
+extern const struct optdesc opt_tcp_signature_enable;
+extern const struct optdesc opt_tcp_abort_threshold;
+extern const struct optdesc opt_tcp_conn_abort_threshold;
+extern const struct optdesc opt_tcp_keepinit;
+extern const struct optdesc opt_tcp_paws;
+extern const struct optdesc opt_tcp_sackena;
+extern const struct optdesc opt_tcp_tsoptena;
 
 extern int xioopen_tcp6_listen(char *a1, int rw, xiofile_t *fd,
 			     unsigned groups, int dummy1, int dummy2,
