@@ -1,5 +1,5 @@
-/* $Id: xio.h,v 1.44 2004/06/20 21:22:59 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2004 */
+/* $Id: xio.h,v 1.46 2005/03/13 12:09:09 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xio_h_included
@@ -10,6 +10,8 @@
 #include "sysutils.h"
 #endif
 
+#define XIO_MAXSOCK 2
+
 /* Linux 2.2.10 */
 #define HAVE_STRUCT_LINGER 1
 
@@ -19,6 +21,23 @@
 
 struct addrdesc;
 
+/* the flags argument of xioopen */
+#define XIO_RDONLY  O_RDONLY /* asserted to be 0 */
+#define XIO_WRONLY  O_WRONLY /* asserted to be 1 */
+#define XIO_RDWR    O_RDWR   /* asserted to be 2 */
+#define XIO_ACCMODE O_ACCMODE	/* must be 3 */
+#define XIO_MAYFORK     4 /* address is allowed to fork the program (fork) */
+#define XIO_MAYCHILD    8 /* address is allowed to fork off a child (exec)*/
+#define XIO_MAYEXEC    16 /* address is allowed to exec a prog (exec+nofork) */
+#define XIO_MAYCONVERT 32 /* address is allowed to perform modifications on the
+			     stream data, e.g. SSL, REALDINE; CRLF */
+
+/* the status flags of xiofile_t */
+#define XIO_DOESFORK    XIO_MAYFORK
+#define XIO_DOESCHILD   XIO_MAYCHILD
+#define XIO_DOESEXEC    XIO_MAYEXEC
+#define XIO_DOESCONVERT XIO_MAYCONVERT
+
 /* these are the values allowed for the "enum xiotag  tag" flag of the "struct
    single" and "union bipipe" (xiofile_t) structures. */
 enum xiotag {
@@ -88,10 +107,14 @@ struct single {
    struct termios savetty;	/* save orig tty settings for later restore */
 #endif /* WITH_TERMIOS */
    const char *name;		/* only with END_UNLINK */
+   int (*sigchild)(struct single *);	/* callback after sigchild */
    union {
       struct {
 	 int fdout;		/* use fd for output */
       } bipipe;
+      struct {
+	 struct timeval connect_timeout; /* how long to hang in connect() */
+      } socket;
       struct {
 	 pid_t pid;		/* child PID, with EXEC: */
 	 int fdout;		/* use fd for output if two pipes */
@@ -111,6 +134,7 @@ struct single {
 #endif /* WITH_READLINE */
 #if WITH_OPENSSL
       struct {
+	 struct timeval connect_timeout; /* how long to hang in connect() */
 	 SSL *ssl;
 	 SSL_CTX* ctx;
       } openssl;
@@ -232,6 +256,7 @@ struct opt {
 } ;
 
 extern const char *PIPESEP;
+extern xiofile_t *sock[XIO_MAXSOCK];
 
 /* return values of xioopensingle */
 #define STAT_OK		0
@@ -248,7 +273,7 @@ extern int xioinitialize(void);
 extern int xiosetopt(char what, const char *arg);
 extern int xioinqopt(char what, char *arg, size_t n);
 extern xiofile_t *xioopen(const char *args, int flags);
-extern int xioopensingle(char *addr, xiofile_t *fd, int rw);
+extern int xioopensingle(char *addr, xiofile_t *fd, int xioflags);
 extern int xioopenhelp(FILE *of, int level);
 
 /* must be outside function for use by childdied handler */
@@ -259,6 +284,7 @@ extern pid_t diedunknown2;
 extern pid_t diedunknown3;
 extern pid_t diedunknown4;
 
+extern int xiosetsigchild(xiofile_t *xfd, int (*callback)(struct single *));
 extern int xio_opt_signal(pid_t pid, int signum);
 extern void childdied(int signum);
 
