@@ -1,5 +1,5 @@
-/* $Id: xioopts.c,v 1.70 2004/12/03 21:53:24 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2004 */
+/* $Id: xioopts.c,v 1.72 2005/03/12 18:03:22 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this file contains the source for address options handling */
@@ -248,6 +248,7 @@ const struct optname optionnames[] = {
 #ifdef TCP_CONN_ABORT_THRESHOLD  /* HP_UX */
 	IF_TCP    ("conn-abort-threshold",	&opt_tcp_conn_abort_threshold)
 #endif
+	IF_SOCKET ("connect-timeout",	&opt_connect_timeout)
 #ifdef TCP_CORK
 	IF_TCP    ("cork",	&opt_tcp_cork)
 #endif
@@ -305,6 +306,8 @@ const struct optname optionnames[] = {
 	IF_SOCKET ("dgram-errind",	&opt_so_dgram_errind)
 	IF_SOCKET ("dgramerrind",	&opt_so_dgram_errind)
 #endif
+	IF_OPENSSL("dh",	&opt_openssl_dhparam)
+	IF_OPENSSL("dhparam",	&opt_openssl_dhparam)
 #ifdef O_DIRECT
 	IF_OPEN   ("direct",	&opt_o_direct)
 #endif
@@ -775,6 +778,7 @@ const struct optname optionnames[] = {
 	IF_OPENSSL("openssl-capath",	&opt_openssl_capath)
 	IF_OPENSSL("openssl-certificate",	&opt_openssl_certificate)
 	IF_OPENSSL("openssl-cipherlist",	&opt_openssl_cipherlist)
+	IF_OPENSSL("openssl-dhparam",	&opt_openssl_dhparam)
 	IF_OPENSSL("openssl-egd",	&opt_openssl_egd)
 	IF_OPENSSL("openssl-key",	&opt_openssl_key)
 	IF_OPENSSL("openssl-method",	&opt_openssl_method)
@@ -2852,7 +2856,7 @@ int applyopts_fchown(int fd, struct opt *opts) {
    return 0;
 }
 
-int applyopts_offset(xiofile_t *xfd, struct opt *opts) {
+int applyopts_offset(struct single *xfd, struct opt *opts) {
    struct opt *opt;
    unsigned char *ptr;
 
@@ -2865,6 +2869,8 @@ int applyopts_offset(xiofile_t *xfd, struct opt *opts) {
       switch (opt->desc->type) {
       case TYPE_DOUBLE:
 	 *(double *)ptr = opt->value.u_double;  break;
+      case TYPE_TIMEVAL:
+	 *(struct timeval *)ptr = opt->value.u_timeval;  break;
       case TYPE_STRING_NULL:
 	 if (opt->value.u_string == NULL) {
 	    *(char **)ptr = NULL;
