@@ -1,5 +1,5 @@
-/* $Id: xioopts.h,v 1.49 2004/12/03 21:53:24 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2004 */
+/* $Id: xioopts.h,v 1.51 2005/03/12 18:02:50 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xioopts_h_included
@@ -199,6 +199,7 @@ enum e_optcode {
    /*OPT_CIBAUD,*/		/* termios.c_cflag */
    OPT_CLOCAL,		/* termios.c_cflag */
    OPT_CLOEXEC,
+   OPT_CONNECT_TIMEOUT,	/* socket connect */
    OPT_CR,		/* customized */
 #ifdef CR0
    OPT_CR0,		/* termios.c_oflag */
@@ -372,6 +373,7 @@ enum e_optcode {
    OPT_OPENSSL_CAPATH,
    OPT_OPENSSL_CERTIFICATE,
    OPT_OPENSSL_CIPHERLIST,
+   OPT_OPENSSL_DHPARAM,
    OPT_OPENSSL_EGD,
    OPT_OPENSSL_KEY,
    OPT_OPENSSL_METHOD,
@@ -780,7 +782,7 @@ extern int applyopts_cloexec(int fd, struct opt *opts);
 extern int applyopts_early(const char *path, struct opt *opts);
 extern int applyopts_fchown(int fd, struct opt *opts);
 extern int applyopts_single(struct single *fd, struct opt *opts, enum e_phase phase);
-extern int applyopts_offset(xiofile_t *xfd, struct opt *opts);
+extern int applyopts_offset(struct single *xfd, struct opt *opts);
 extern int applyopts_signal(struct single *xfd, struct opt *opts);
 extern int _xio_openlate(struct single *fd, struct opt *opts);
 extern int parseopts(char *a, unsigned int groups, struct opt **opts);
