@@ -1,5 +1,5 @@
-/* $Id: xioshutdown.c,v 1.17 2004/09/18 12:14:12 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2004 */
+/* $Id: xioshutdown.c,v 1.18 2005/03/13 12:19:59 gerhard Exp $ */
+/* Copyright Gerhard Rieger 2001-2005 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this is the source of the extended shutdown function */
@@ -77,7 +77,7 @@ int xioshutdown(xiofile_t *sock, int how) {
 	 Warn3("shutdown(%d, %d): %s",
 	       sock->stream.fd, how, strerror(errno));
       }
-      if (sock->stream.flags == O_WRONLY) {
+      if ((sock->stream.flags&XIO_ACCMODE) == XIO_WRONLY) {
 	 /* the child process might want to flush some data before terminating
 	    */
 	 int status = 0;
